@@ -97,7 +97,7 @@ macro_rules! opcode {
 /// inline zeroed to improve codegen
 #[inline(always)]
 fn sqe_zeroed() -> sys::io_uring_sqe {
-    unsafe { std::mem::zeroed() }
+    unsafe { mem::zeroed() }
 }
 
 opcode! {
@@ -871,7 +871,7 @@ opcode! {
         /// `offset` contains the read or write offset.
         ///
         /// If `fd` does not refer to a seekable file, `offset` must be set to zero.
-        /// If `offsett` is set to `-1`, the offset will use (and advance) the file position,
+        /// If `offset` is set to `-1`, the offset will use (and advance) the file position,
         /// like the `read(2)` and `write(2)` system calls.
         offset: u64 = 0,
         ioprio: u16 = 0,
@@ -1453,9 +1453,9 @@ opcode! {
         sqe.len = result as u32;
         sqe.__bindgen_anon_1.off = user_data;
         sqe.__bindgen_anon_3.msg_ring_flags = opcode_flags;
-        if let Some(_flags) = user_flags {
-            // TODO(lucab): add IORING_MSG_RING_FLAGS_PASS support (in v6.3):
-            // https://lore.kernel.org/all/20230103160507.617416-1-leitao@debian.org/t/#u
+        if let Some(flags) = user_flags {
+            sqe.__bindgen_anon_5.file_index = flags;
+            unsafe {sqe.__bindgen_anon_3.msg_ring_flags |= sys::IORING_MSG_RING_FLAGS_PASS};
         }
         Entry(sqe)
     }
@@ -1463,12 +1463,38 @@ opcode! {
 
 // === 5.19 ===
 
+opcode! {
+    /// Attempt to cancel an already issued request, receiving a cancellation
+    /// builder, which allows for the new cancel criterias introduced since
+    /// 5.19.
+    pub struct AsyncCancel2 {
+        builder: { types::CancelBuilder }
+        ;;
+    }
+
+    pub const CODE = sys::IORING_OP_ASYNC_CANCEL;
+
+    pub fn build(self) -> Entry {
+        let AsyncCancel2 { builder } = self;
+
+        let mut sqe = sqe_zeroed();
+        sqe.opcode = Self::CODE;
+        sqe.fd = builder.to_fd();
+        sqe.__bindgen_anon_2.addr = builder.user_data.unwrap_or(0);
+        sqe.__bindgen_anon_3.cancel_flags = builder.flags.bits();
+        Entry(sqe)
+    }
+}
+
 opcode! {
     /// A file/device-specific 16-byte command, akin (but not equivalent) to `ioctl(2)`.
     pub struct UringCmd16 {
         fd: { impl sealed::UseFixed },
         cmd_op: { u32 },
         ;;
+        /// The `buf_index` is an index into an array of fixed buffers,
+        /// and is only valid if fixed buffers were registered.
+        buf_index: Option<u16> = None,
         /// Arbitrary command data.
         cmd: [u8; 16] = [0u8; 16]
     }
@@ -1476,13 +1502,19 @@ opcode! {
     pub const CODE = sys::IORING_OP_URING_CMD;
 
     pub fn build(self) -> Entry {
-        let UringCmd16 { fd, cmd_op, cmd } = self;
+        let UringCmd16 { fd, cmd_op, cmd, buf_index } = self;
 
         let mut sqe = sqe_zeroed();
         sqe.opcode = Self::CODE;
         assign_fd!(sqe.fd = fd);
         sqe.__bindgen_anon_1.__bindgen_anon_1.cmd_op = cmd_op;
         unsafe { *sqe.__bindgen_anon_6.cmd.as_mut().as_mut_ptr().cast::<[u8; 16]>() = cmd };
+        if let Some(buf_index) = buf_index {
+            sqe.__bindgen_anon_4.buf_index = buf_index;
+            unsafe {
+                sqe.__bindgen_anon_3.uring_cmd_flags |= sys::IORING_URING_CMD_FIXED;
+            }
+        }
         Entry(sqe)
     }
 }
@@ -1493,6 +1525,9 @@ opcode! {
         fd: { impl sealed::UseFixed },
         cmd_op: { u32 },
         ;;
+        /// The `buf_index` is an index into an array of fixed buffers,
+        /// and is only valid if fixed buffers were registered.
+        buf_index: Option<u16> = None,
         /// Arbitrary command data.
         cmd: [u8; 80] = [0u8; 80]
     }
@@ -1500,7 +1535,7 @@ opcode! {
     pub const CODE = sys::IORING_OP_URING_CMD;
 
     pub fn build(self) -> Entry128 {
-        let UringCmd80 { fd, cmd_op, cmd } = self;
+        let UringCmd80 { fd, cmd_op, cmd, buf_index } = self;
 
         let cmd1 = cmd[..16].try_into().unwrap();
         let cmd2 = cmd[16..].try_into().unwrap();
@@ -1510,6 +1545,12 @@ opcode! {
         assign_fd!(sqe.fd = fd);
         sqe.__bindgen_anon_1.__bindgen_anon_1.cmd_op = cmd_op;
         unsafe { *sqe.__bindgen_anon_6.cmd.as_mut().as_mut_ptr().cast::<[u8; 16]>() = cmd1 };
+        if let Some(buf_index) = buf_index {
+            sqe.__bindgen_anon_4.buf_index = buf_index;
+            unsafe {
+                sqe.__bindgen_anon_3.uring_cmd_flags |= sys::IORING_URING_CMD_FIXED;
+            }
+        }
         Entry128(Entry(sqe), cmd2)
     }
 }
@@ -1694,3 +1735,102 @@ opcode! {
         Entry(sqe)
     }
 }
+
+// === 6.7 ===
+
+opcode! {
+    /// Wait on a futex, like but not equivalant to `futex(2)`'s `FUTEX_WAIT_BITSET`.
+    ///
+    /// Wait on a futex at address `futex` and which still has the value `val` and with `futex2(2)`
+    /// flags of `futex_flags`. `musk` can be set to a specific bitset mask, which will be matched
+    /// by the waking side to decide who to wake up. To always get woken, an application may use
+    /// `FUTEX_BITSET_MATCH_ANY` (truncated to futex bits). `futex_flags` follows the `futex2(2)`
+    /// flags, not the `futex(2)` v1 interface flags. `flags` are currently unused and hence `0`
+    /// must be passed.
+    #[derive(Debug)]
+    pub struct FutexWait {
+        futex: { *const u32 },
+        val: { u64 },
+        mask: { u64 },
+        futex_flags: { u32 },
+        ;;
+        flags: u32 = 0
+    }
+
+    pub const CODE = sys::IORING_OP_FUTEX_WAIT;
+
+    pub fn build(self) -> Entry {
+        let FutexWait { futex, val, mask, futex_flags, flags } = self;
+
+        let mut sqe = sqe_zeroed();
+        sqe.opcode = Self::CODE;
+        sqe.fd = futex_flags as _;
+        sqe.__bindgen_anon_2.addr = futex as usize as _;
+        sqe.__bindgen_anon_1.off = val;
+        unsafe { sqe.__bindgen_anon_6.__bindgen_anon_1.as_mut().addr3 = mask };
+        sqe.__bindgen_anon_3.futex_flags = flags;
+        Entry(sqe)
+    }
+}
+
+opcode! {
+    /// Wake up waiters on a futex, like but not equivalant to `futex(2)`'s `FUTEX_WAKE_BITSET`.
+    ///
+    /// Wake any waiters on the futex indicated by `futex` and at most `val` futexes. `futex_flags`
+    /// indicates the `futex2(2)` modifier flags. If a given bitset for who to wake is desired,
+    /// then that must be set in `mask`. Use `FUTEX_BITSET_MATCH_ANY` (truncated to futex bits) to
+    /// match any waiter on the given futex. `flags` are currently unused and hence `0` must be
+    /// passed.
+    #[derive(Debug)]
+    pub struct FutexWake {
+        futex: { *const u32 },
+        val: { u64 },
+        mask: { u64 },
+        futex_flags: { u32 },
+        ;;
+        flags: u32 = 0
+    }
+
+    pub const CODE = sys::IORING_OP_FUTEX_WAKE;
+
+    pub fn build(self) -> Entry {
+        let FutexWake { futex, val, mask, futex_flags, flags } = self;
+
+        let mut sqe = sqe_zeroed();
+        sqe.opcode = Self::CODE;
+        sqe.fd = futex_flags as _;
+        sqe.__bindgen_anon_2.addr = futex as usize as _;
+        sqe.__bindgen_anon_1.off = val;
+        unsafe { sqe.__bindgen_anon_6.__bindgen_anon_1.as_mut().addr3 = mask };
+        sqe.__bindgen_anon_3.futex_flags = flags;
+        Entry(sqe)
+    }
+}
+
+opcode! {
+    /// Wait on multiple futexes.
+    ///
+    /// Wait on multiple futexes at the same time. Futexes are given by `futexv` and `nr_futex` is
+    /// the number of futexes in that array. Unlike `FutexWait`, the desired bitset mask and values
+    /// are passed in `futexv`. `flags` are currently unused and hence `0` must be passed.
+    #[derive(Debug)]
+    pub struct FutexWaitV {
+        futexv: { *const types::FutexWaitV },
+        nr_futex: { u32 },
+        ;;
+        flags: u32 = 0
+    }
+
+    pub const CODE = sys::IORING_OP_FUTEX_WAITV;
+
+    pub fn build(self) -> Entry {
+        let FutexWaitV { futexv, nr_futex, flags } = self;
+
+        let mut sqe = sqe_zeroed();
+        sqe.opcode = Self::CODE;
+        sqe.__bindgen_anon_2.addr = futexv as usize as _;
+        sqe.len = nr_futex;
+        sqe.__bindgen_anon_3.futex_flags = flags;
+        Entry(sqe)
+    }
+}
