@@ -43,6 +43,7 @@ pub(crate) mod sealed {
 use crate::sys;
 use crate::util::{cast_ptr, unwrap_nonzero, unwrap_u32};
 use bitflags::bitflags;
+use std::convert::TryFrom;
 use std::marker::PhantomData;
 use std::num::NonZeroU32;
 use std::os::unix::io::RawFd;
@@ -51,12 +52,14 @@ pub use sys::__kernel_rwf_t as RwFlags;
 
 /// Opaque types, you should use [`statx`](struct@libc::statx) instead.
 #[repr(C)]
+#[allow(non_camel_case_types)]
 pub struct statx {
     _priv: (),
 }
 
 /// Opaque types, you should use [`epoll_event`](libc::epoll_event) instead.
 #[repr(C)]
+#[allow(non_camel_case_types)]
 pub struct epoll_event {
     _priv: (),
 }
@@ -193,6 +196,14 @@ impl Timespec {
     }
 }
 
+impl From<std::time::Duration> for Timespec {
+    fn from(value: std::time::Duration) -> Self {
+        Timespec::new()
+            .sec(value.as_secs())
+            .nsec(value.subsec_nanos())
+    }
+}
+
 /// Submit arguments
 ///
 /// Note that arguments that exceed their lifetime will fail to compile.
@@ -367,10 +378,7 @@ pub struct RecvMsgOut<'buf> {
     /// If it is smaller, it gets 0-padded to fill the whole field. In either case,
     /// this fixed amount of space is reserved in the result buffer.
     msghdr_name_len: usize,
-    /// The fixed length of the control field, in bytes.
-    ///
-    /// This follows the same semantics as the field above, but for control data.
-    msghdr_control_len: usize,
+
     name_data: &'buf [u8],
     control_data: &'buf [u8],
     payload_data: &'buf [u8],
@@ -386,7 +394,15 @@ impl<'buf> RecvMsgOut<'buf> {
     /// (only `msg_namelen` and `msg_controllen` fields are relevant).
     #[allow(clippy::result_unit_err)]
     pub fn parse(buffer: &'buf [u8], msghdr: &libc::msghdr) -> Result<Self, ()> {
-        if buffer.len() < std::mem::size_of::<sys::io_uring_recvmsg_out>() {
+        let msghdr_name_len = usize::try_from(msghdr.msg_namelen).unwrap();
+        let msghdr_control_len = usize::try_from(msghdr.msg_controllen).unwrap();
+
+        if Self::DATA_START
+            .checked_add(msghdr_name_len)
+            .and_then(|acc| acc.checked_add(msghdr_control_len))
+            .map(|header_len| buffer.len() < header_len)
+            .unwrap_or(true)
+        {
             return Err(());
         }
         // SAFETY: buffer (minimum) length is checked here above.
@@ -397,45 +413,36 @@ impl<'buf> RecvMsgOut<'buf> {
                 .read_unaligned()
         };
 
-        let msghdr_name_len = msghdr.msg_namelen as _;
-        let msghdr_control_len = msghdr.msg_controllen as _;
-
-        // Check total length upfront, so that further logic here
-        // below can safely use unchecked/saturating math.
-        let length_overflow = Some(Self::DATA_START)
-            .and_then(|acc| acc.checked_add(msghdr_name_len))
-            .and_then(|acc| acc.checked_add(msghdr_control_len))
-            .and_then(|acc| acc.checked_add(header.payloadlen as usize))
-            .map(|total_len| total_len > buffer.len())
-            .unwrap_or(true);
-        if length_overflow {
-            return Err(());
-        }
-
+        // min is used because the header may indicate the true size of the data
+        // while what we received was truncated.
         let (name_data, control_start) = {
             let name_start = Self::DATA_START;
-            let name_size = usize::min(header.namelen as usize, msghdr_name_len);
-            let name_data_end = name_start.saturating_add(name_size);
-            let name_data = &buffer[name_start..name_data_end];
-            let name_field_end = name_start.saturating_add(msghdr_name_len);
-            (name_data, name_field_end)
+            let name_data_end =
+                name_start + usize::min(usize::try_from(header.namelen).unwrap(), msghdr_name_len);
+            let name_field_end = name_start + msghdr_name_len;
+            (&buffer[name_start..name_data_end], name_field_end)
         };
         let (control_data, payload_start) = {
-            let control_size = usize::min(header.controllen as usize, msghdr_control_len);
-            let control_data_end = control_start.saturating_add(control_size);
-            let control_data = &buffer[control_start..control_data_end];
-            let control_field_end = control_start.saturating_add(msghdr_control_len);
-            (control_data, control_field_end)
+            let control_data_end = control_start
+                + usize::min(
+                    usize::try_from(header.controllen).unwrap(),
+                    msghdr_control_len,
+                );
+            let control_field_end = control_start + msghdr_control_len;
+            (&buffer[control_start..control_data_end], control_field_end)
         };
         let payload_data = {
-            let payload_data_end = payload_start.saturating_add(header.payloadlen as usize);
+            let payload_data_end = payload_start
+                + usize::min(
+                    usize::try_from(header.payloadlen).unwrap(),
+                    buffer.len() - payload_start,
+                );
             &buffer[payload_start..payload_data_end]
         };
 
         Ok(Self {
             header,
             msghdr_name_len,
-            msghdr_control_len,
             name_data,
             control_data,
             payload_data,
@@ -480,7 +487,7 @@ impl<'buf> RecvMsgOut<'buf> {
     /// When `true`, data returned by `control_data()` is truncated and
     /// incomplete.
     pub fn is_control_data_truncated(&self) -> bool {
-        self.header.controllen as usize > self.msghdr_control_len
+        (self.header.flags & u32::try_from(libc::MSG_CTRUNC).unwrap()) != 0
     }
 
     /// Message control data, with the same semantics as `msghdr.msg_control`.
@@ -493,7 +500,7 @@ impl<'buf> RecvMsgOut<'buf> {
     /// When `true`, data returned by `payload_data()` is truncated and
     /// incomplete.
     pub fn is_payload_truncated(&self) -> bool {
-        self.header.flags & (libc::MSG_TRUNC as u32) != 0
+        (self.header.flags & u32::try_from(libc::MSG_TRUNC).unwrap()) != 0
     }
 
     /// Message payload, as buffered by the kernel.
@@ -501,6 +508,16 @@ impl<'buf> RecvMsgOut<'buf> {
         self.payload_data
     }
 
+    /// Return the length of the incoming `payload` data.
+    ///
+    /// This may be larger than the size of the content returned by
+    /// `payload_data()`, if the kernel could not fit all the incoming
+    /// data in the provided buffer size. In that case, payload data in
+    /// the result buffer gets truncated.
+    pub fn incoming_payload_len(&self) -> u32 {
+        self.header.payloadlen
+    }
+
     /// Message flags, with the same semantics as `msghdr.msg_flags`.
     pub fn flags(&self) -> u32 {
         self.header.flags
@@ -543,6 +560,8 @@ impl CancelBuilder {
     /// Create a new [CancelBuilder] which will match any in-flight request.
     ///
     /// This will cancel every in-flight request in the ring.
+    ///
+    /// Async cancellation matching any requests is only available since 5.19.
     pub const fn any() -> Self {
         Self {
             flags: AsyncCancelFlags::ANY,
@@ -570,6 +589,8 @@ impl CancelBuilder {
     ///
     /// The first request with the given `fd` value will be canceled. [CancelBuilder::all](#method.all)
     /// can be called to instead match every request with the provided `fd` value.
+    ///
+    /// FD async cancellation is only available since 5.19.
     pub fn fd(fd: impl sealed::UseFixed) -> Self {
         let mut flags = AsyncCancelFlags::FD;
         let target = fd.into();
@@ -587,18 +608,73 @@ impl CancelBuilder {
     /// rather than just the first one.
     ///
     /// This has no effect when combined with [CancelBuilder::any](#method.any).
+    ///
+    /// Async cancellation matching all requests is only available since 5.19.
     pub fn all(mut self) -> Self {
         self.flags.insert(AsyncCancelFlags::ALL);
         self
     }
+
+    pub(crate) fn to_fd(&self) -> i32 {
+        self.fd
+            .as_ref()
+            .map(|target| match *target {
+                sealed::Target::Fd(fd) => fd,
+                sealed::Target::Fixed(idx) => idx as i32,
+            })
+            .unwrap_or(-1)
+    }
+}
+
+/// Wrapper around `futex_waitv` as used in [`futex_waitv` system
+/// call](https://www.kernel.org/doc/html/latest/userspace-api/futex2.html).
+#[derive(Default, Debug, Clone, Copy)]
+#[repr(transparent)]
+pub struct FutexWaitV(sys::futex_waitv);
+
+impl FutexWaitV {
+    pub const fn new() -> Self {
+        Self(sys::futex_waitv {
+            val: 0,
+            uaddr: 0,
+            flags: 0,
+            __reserved: 0,
+        })
+    }
+
+    pub const fn val(mut self, val: u64) -> Self {
+        self.0.val = val;
+        self
+    }
+
+    pub const fn uaddr(mut self, uaddr: u64) -> Self {
+        self.0.uaddr = uaddr;
+        self
+    }
+
+    pub const fn flags(mut self, flags: u32) -> Self {
+        self.0.flags = flags;
+        self
+    }
 }
 
 #[cfg(test)]
 mod tests {
+    use std::time::Duration;
+
     use crate::types::sealed::Target;
 
     use super::*;
 
+    #[test]
+    fn timespec_from_duration_converts_correctly() {
+        let duration = Duration::new(2, 500);
+        let timespec = Timespec::from(duration);
+
+        assert_eq!(timespec.0.tv_sec as u64, duration.as_secs());
+        assert_eq!(timespec.0.tv_nsec as u32, duration.subsec_nanos());
+    }
+
     #[test]
     fn test_cancel_builder_flags() {
         let cb = CancelBuilder::any();
